// com.h, Headerdatei zu com.cpp,
// Benutzung der seriellen Schnitsttelle unter
// Borland-C++ Builder V 4.0
// sa/hhs, 19.12.2000
//
// nderungen:
//   24.10.99: Funktionen "SendeString" und "HoleString" ergnzt
//
//   02.04.00: Abfragefunktionen fr Meldeleitungen
//             CTSActive(), DSRActive() und RIActive ergnzt
//   19.12.00: Funktionen ClrRts, SetRts, ClrDtr, SetDtr zum Setzen/Lschen
//             der Steuerleitungen ergnzt
// *******************************************************************
// Benutzungsanleitung:
// 1. Dateien com.h und com.cpp ins Projektverzechnis kopieren
//
// 2. Mit Menu "Projekt / Hinzufgen"  die Datei com.cpp in das
//    aktuelle Projekt aufnehmen.
//
// 3. In die Formulardatei, die die serielle Schnittstelle benutzt,
//    (z.B. form1.cpp), die Headerdatei aufnehmen (#include "com.h")
//
// 4. Funktionen benutzen,  wie unten in Beispielen genannt.
//
// Grundstzliche Reihenfolge:
//   - Erst Schnittstelle ffnen ("Oeffneseriell"),
//   - Dann benutzen ("SendeZeichen","PufferLeeren",
//                    "ZeichenImEmpfangspuffer", "HoleZeichen"  und andere
//                     Funktionen)
//   - Zuletzt schliessen ("SchliesseSeriell")
//
// In der vorliegenden Version ist keinerlei Handshake eingestellt
//  (weder RTS / CTS  noch DTR / DSR  noch XON / XOFF).
// Es gilt:
//
//    Die Steuerleitungen RTS  und DTR sind nach Aufruf von "OeffneSeriell"
//    aktiv  (+6V...+12V), knnen aber durch Aufruf der Funktionen ClrRts,
//    SetRts, ClrDtr, SetDtr beeinflusst werden (siehe unten).
//
//    Da kein Hardware-Handshake eingestellt ist, werden die Meldeleitungen
//    CTS und DSR ignoriert, knnen aber, wie auch RI, abgefragt werden
//    (siehe unten).
//
//    Da kein Software-Handshake eingestellt ist, werden empfangene
//    Steuerzeichen XON / XOFF wie auch alle anderen Zeichen an die benutzende
//    Schicht weitergereicht.
//
//  Bei Bedarf kann Hard- und/oder Softearehandshake eingestellt werden.
//  Siehe Datei com.cpp, Behandlung des Gertesteuerblockes "mydcb"
//  in der Funktion "OeffneSeriell". Beachten Sie aber, da Sie dann die Steuer-
//  und Meldeleitungen nicht mehr selbst setzen/lschen und abfragen drfen !
//
//  Zustzliche Information dazu finden Sie in der Zeitschrift
//  toolbox 5 / 97, Artikel "Windows und die Bytes im Gnsemarsch"
//  bzw. im Buch "Borland C++ Builder 3", ISBN 3-932311-24-8.
//
//  Siehe auch Beispielprogramm "serdemo"
// *******************************************************************
//---------------------------------------------------------------------------
#ifndef comH
#define comH
//---------------------------------------------------------------------------
// Der unten verwendete Typ DWORD (aus WIN-API) entspricht dem C++ Standard-Typ
// "unsigned long", also einer 32-Bit-Zahl ohne Vorzeichen.
//
// Der Typ BOOL (aus Win-API) entspricht dem Typ bool aus
// neuerem C++ Standard.
//

BOOL OeffneSeriell(int comnummer,int baudrate,
                   byte bitsprozeichen,byte paritaet, int stopbits);
 // ffnet serielle Schnittstelle mit Parametern.
 // Baudrate(je nach UART-Baustein, evtl. probieren):
 //     110,300,600,1200,2400,4800,9600,14400,19200,38400,57600,115200.
 //     Siehe auch "winbase.h", Konstantendeklarationen  CBR_110  usw.
 //   ComNummer:1..4
 //             (je nach Ausstattung. Achtung ! Evtl. Probleme mit
 //              Interrupt-Sharing zwischen COM1 und 3 bzw.
 //              zwischen COM2 und 4)
 //   BitsProZeichen: 5, 7 oder 8
 //   Paritt:  0:keine
 //             1: ungerade
 //             2: gerade
 //             3: mark (immer 1)
 //             4: space (immer 0)
 //   Stopbits  0: 1 Stopbit
 //             1: 1.5 Stopbits (nur bei 5 Bit pro Zeichen mglich)
 //             2: 2 Stopbits
 //
 //Aufrufbeispiel:
 // if  (!OeffneSeriell(2,9600,8,0,0) ) //COM2,9600 Baud,keine Paritt,1 Stopbit
 //   ShowMessage ("Kann COM nicht ffnen");

DWORD SendeZeichen(char &puffer, DWORD anzahl);
// sendet eine Anzahl von Zeichen ("anzahl") ab einer Startadresse,
// die durch die Referenz  "&puffer" gegeben ist.
// Im "Gut"-Fall ist der Rckgabewert die Anzahl der gesendeten Zeichen,
// andernfalls 0.
// Aufrufbeispiel:
//
// char satz[6]= "abcde"; DWORD i;
// i=SendeZeichen(satz[0],5);
// if (i==0) ShowMessage("Fehler");


DWORD SendeString(AnsiString &s);
// sendet den gesamten Inhalt des AnsiStrings s. Rckgabewert wie
// bei "SendeZeichen": Anzahl der gesendeten Zeichen.
// Aufrufbeispiel:
//
// i = SendeString (Edit1->Text);
// if (i != Edit1->Text.Length()) ShowMessage("Fehler");


DWORD ZeichenImEmpfangspuffer(void);
// gibt die Anzahl der Zeichen zurck, die im Empfangspuffer der aktuellen
// seriellen Schnittstelle gespeichert sind.
// Wert 0 kann entweder bedeuten, dass kein Zeichen empfangen wurde oder
// dass Schnittstelle nicht korrekt geffnet ist.
// Aufrufbeispiel siehe folgende Funktion

DWORD HoleZeichen(char &puffer, DWORD anzahl);
// liest eine Anzahl von Zeichen ("anzahl") in einen Speicherbereich, der
// durch die Referenz "&puffer" gegeben ist.
// "anzahl" darf nicht grsser als die tatschlich vorhandene Anzahl von
// Zeichen im Empfangspuffer sein (evtl. zuvor mit "ZeichenImEmpfangspuffer"
// erfragen). Das Funktionsergebnis ist die tatschlich gelesene Zahl von
// Zeichen oder, im Fehlerfall, der Wert 0.
//
// Aufrufbeispiel: in einer Timer-Ereignisbehandlung wird geprft, wieviele
// Zeichen vorrtig sind. Diese werden zuerst in einen Hilfspuffer geholt
// und von dort aus an ein Memo angehngt. )
//
//   DWORD i,anzahl; char puffer [1024];
//   anzahl = ZeichenImEmpfangspuffer();
//   if (anzahl>0 && anzahl <=1024)
//     {
//      anzahl=HoleZeichen(puffer[0],anzahl);
//      for (i=0;i<anzahl;i++)
//      Memo1->Text = Memo1->Text+puffer[i];
//     }

DWORD HoleString(AnsiString &s);
// holt alle verfgbaren Zeichen in den AnsiString s. Aufruf ist auch erlaubt,
// wenn kein Zeichen im Empfangspuffer ist. Dann erhlt s die Lnge 0.
// Aufrufbeispiel:
//
//  AnsiString x;
//  HoleZeichen (x);
//  Memo1->Text = x;
//

BOOL PufferLeeren(void);
// Rumt die Schnittstelle aus (lscht Sende- und Empfangspuffer).
// Sinnvoll z.B., um einen Dialog mit einem anderen Gert neu aufzusetzen,
// nachdem ein Timeout o.. auftrat. Erspart dann, Schnittstelle zu schliessen
// und neu zu ffnen.
// Ergebnis normalerweise TRUE, ausser, wenn Schnittstelle nicht korrekt
// geffnet wurde.
// Aufrufbeispiel:
//  if ( !PufferLeeren() )
//    ShowMessage
//     ("Wozu leeren Sie den Puffer, wenn die Schniuttstelle nicht geffnet ist ?");
//

//********************************************
//******* Setzen/Lschen von Steuerleitungen:
//********************************************

void SetDtr (void);
// setzt DTR = High (+12V)
// DTR ist Pin 20 (25-pol.) bzw Pin 4 (9-pol.)
//Aufrufbeispiel:
//  SetDtr();

void ClrDtr(void);
// Setzt DTR = Low (-12V)
// DTR ist Pin 20 (25-pol.) bzw Pin 4 (9-pol.)
//Aufrufbeispiel:
//  ClrtDtr();

void SetRts (void);
// Setzt RTS = High (+12V)
// RTS ist Pin 4 (25-pol.) bzw Pin 7 (9-pol.)
//Aufrufbeispiel:
//  SetRts();

void ClrRts (void);
// Setzt RTS = Low (-12V)
// RTS ist Pin 4 (25-pol.) bzw Pin 7 (9-pol.)
//Aufrufbeispiel:
//  ClrRts();


//********************************************
//************ Abfragen von Meldeleitungen:
//********************************************

BOOL CTSActive(void);
// liefert true, wenn Leitung CTS der Schnittstelle aktiv, (positive Logik,
// +6V .. +15V lt. Spezifikation)
// anderfalls  (-6V .. -15V) false.
// Verhalten bei ungeffneter Schnittstelle undefiniert.
// CTS ist Pin 5 (25-pol.) bzw Pin 8 (9-pol.)
//
// Aufrufbeispiel:
//  if ( CTSActive() ) ...........


BOOL DSRActive(void);
// liefert true, wenn Leitung DSR der Schnittstelle aktiv, (positive Logik,
// +6V .. +15V lt. Spezifikation)
// anderfalls  (-6V .. -15V) false.
// Verhalten bei ungeffneter Schnittstelle undefiniert.
// DSR ist Pin 6 bei 25-pol. und bei 9-pol. Schnittstelle
//
// Aufrufbeispiel:
//  if ( DSRActive() ) ...........


BOOL RIActive(void);
// liefert true, wenn Leitung RI der Schnittstelle aktiv, (positive Logik,
// +6V .. +15V lt. Spezifikation)
// anderfalls  (-6V .. -15V) false.
// Verhalten bei ungeffneter Schnittstelle undefiniert.
// RI ist Pin 22 (25-pol.) bzw Pin 9 (9-pol.)
//
// Aufrufbeispiel:
//  if ( RIActive() ) ...........


BOOL DCDActive(void);
// liefert true, wenn Leitung DCD der Schnittstelle aktiv, (positive Logik,
// +6V .. +15V lt. Spezifikation)
// anderfalls  (-6V .. -15V) false.
// Verhalten bei ungeffneter Schnittstelle undefiniert.
// DCD ist Pin 8 (25-pol.) bzw Pin 1 (9-pol.)
//
// Aufrufbeispiel:
//  if ( DCDActive() ) ...........


void SchliesseSeriell(void);
// schliesst Schnitsttelle wieder.
// Aufrufbeispiel:
//   SchliesseSeriell();


#endif

